@@ -86,9 +86,18 @@ npplreg.formula <-
                    ifelse(has.eval,paste("exdat = exdat, ezdat = ezdat,",
                                          ifelse(has.ey,"eydat = eydat,","")),""),
                    "bws = bws, ...)")))
-    ev$rows.omit <- as.vector(attr(umf,"na.action"))
+
+    ev$omit <- attr(umf,"na.action")
+    ev$rows.omit <- as.vector(ev$omit)
     ev$nobs.omit <- length(ev$rows.omit)
-    ev
+
+    ev$mean <- napredict(ev$omit, ev$mean)
+    ev$merr <- napredict(ev$omit, ev$merr)
+
+    if(ev$residuals){
+        ev$resid <- naresid(ev$omit, ev$resid)
+    }    
+    return(ev)
   }
 
 npplreg.call <-
@@ -210,7 +219,7 @@ npplreg.plbandwidth <-
 
     Bvcov = sum((tmp.ty-(mmy$mean + resx  %*% B))^2)/
       (dim(txdat)[1]-dim(txdat)[2]-dim(tzdat)[2])*
-      solve(t(model.matrix(model))%*%model.matrix(model))
+      chol2inv(chol(t(model.matrix(model))%*%model.matrix(model)))
 
     Berr = sqrt(diag(Bvcov))
 
@@ -254,7 +263,8 @@ npplreg.plbandwidth <-
 
 
 npplreg.default <- function(bws, txdat, tydat, tzdat, ...) {
-  sc.names <- names(sys.call())
+  sc <- sys.call()
+  sc.names <- names(sc)
 
   ## here we check to see if the function was called with tdat =
   ## if it was, we need to catch that and map it to dat =
@@ -281,44 +291,25 @@ npplreg.default <- function(bws, txdat, tydat, tzdat, ...) {
   if(tydat.named)
     tzdat <- toFrame(tzdat)
 
-  mc <- match.call()
-
-  tx.str <- ifelse(txdat.named, "xdat = txdat,",
-                   ifelse(no.txdat, "", "txdat,"))
-  ty.str <- ifelse(tydat.named, "ydat = tydat,",
-                   ifelse(no.tydat, "", "tydat,"))
-  tz.str <- ifelse(tzdat.named, "zdat = tzdat,",
-                   ifelse(no.tzdat, "", "tzdat,"))
+  sc.bw <- sc
   
-  tbw <- eval(parse(text = paste("npplregbw(",
-                      ifelse(bws.named,                             
-                             paste(tx.str, ty.str, tz.str,
-                                   "bws = bws, bandwidth.compute = FALSE,"),
-                             paste(ifelse(no.bws, "", "bws,"),
-                                   tx.str, ty.str, tz.str)),
-                      "call = mc, ...",")",sep="")))
+  sc.bw[[1]] <- quote(npplregbw)
 
-  ## need to do some surgery on the call to
-  ## allow it to work with the formula interface
-
-  repair.args <- c("data", "subset", "na.action")
-  
-  m.par <- match(repair.args, names(mc), nomatch = 0)
-  m.child <- match(repair.args, names(tbw$call), nomatch = 0)
-
-  if(any(m.child > 0)) {
-    tbw$call[m.child] <- mc[m.par]
+  if(bws.named){
+    sc.bw$bandwidth.compute <- FALSE
   }
 
-  ## next we repair arguments portion of the call
-  m.bws.par <- match(c("bws","txdat","tydat","tzdat"), names(mc), nomatch = 0)
-  m.bws.child <- match(c("bws","txdat","tydat","tzdat"), as.character(tbw$call), nomatch = 0)
-  m.bws.union <- (m.bws.par > 0) & (m.bws.child > 0)
+  ostxy <- c('txdat','tydat','tzdat')
+  nstxy <- c('xdat','ydat','zdat')
   
-  tbw$call[m.bws.child[m.bws.union]] <- mc[m.bws.par[m.bws.union]]
-
-  environment(tbw$call) <- parent.frame()
+  m.txy <- match(ostxy, names(sc.bw), nomatch = 0)
 
+  if(any(m.txy > 0)) {
+    names(sc.bw)[m.txy] <- nstxy[m.txy > 0]
+  }
+    
+  tbw <- eval.parent(sc.bw)
+  
   ## convention: drop 'bws' and up to three unnamed arguments (including bws)
   ## for simplicity, we don't allow for inconsistent
   ## mixes of named/unnamed arguments
