@@ -12,26 +12,18 @@ npindexbw <-
 
 npindexbw.formula <-
   function(formula, data, subset, na.action, call, ...){
-    
+
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"),
                names(mf), nomatch = 0)
     mf <- mf[c(1,m)]
 
-    if(!missing(call) && is.call(call)){
-      formula.args <- c("data", "subset", "na.action")
-      mc.call <- match(formula.args, names(call), nomatch = 0)
-      mc.mf <- match(formula.args, names(mf), nomatch = 0)
-      if(any(mc.mf > 0))
-        mf[mc.mf] <- call[mc.call]
-    }
-
     mf[[1]] <- as.name("model.frame")
     mf <- eval(mf, parent.frame())
-    
+
     ydat <- model.response(mf)
     xdat <- mf[, attr(attr(mf, "terms"),"term.labels"), drop = FALSE]
-    
+
     tbw <- npindexbw(xdat = xdat, ydat = ydat, ...)
 
     ## clean up (possible) inconsistencies due to recursion ...
@@ -51,11 +43,10 @@ npindexbw.formula <-
     tbw
   }
 
-
-npindexbw.NULL <- function(xdat = stop("training data xdat missing"),
-                           ydat = stop("training data ydat missing"),
-                           bws,
-                           ...){
+npindexbw.NULL <-
+  function(xdat = stop("training data xdat missing"),
+           ydat = stop("training data ydat missing"),
+           bws, ...){
 
     xdat <- toFrame(xdat)
 
@@ -63,8 +54,7 @@ npindexbw.NULL <- function(xdat = stop("training data xdat missing"),
 
     tbw <- npindexbw.default(xdat = xdat,
                              ydat = ydat,
-                             bws = bws,
-                             ...)
+                             bws = bws, ...)
 
     ## clean up (possible) inconsistencies due to recursion ...
     mc <- match.call(expand.dots = FALSE)
@@ -77,23 +67,13 @@ npindexbw.NULL <- function(xdat = stop("training data xdat missing"),
 
     tbw
   }
-  
 
 npindexbw.default <-
   function(xdat = stop("training data xdat missing"),
            ydat = stop("training data ydat missing"),
-           bws,
-           bandwidth.compute = TRUE,
-           nmulti,
-           random.seed,
-           optim.method,
-           optim.maxattempts,
-           optim.reltol,
-           optim.abstol,
-           optim.maxit,
-           ckertype,
-           ckerorder,
-           ...){
+           bws, bandwidth.compute = TRUE,
+           nmulti, random.seed, optim.method, optim.maxattempts,
+           optim.reltol, optim.abstol, optim.maxit, only.optimize.beta, ...){
 
     xdat <- toFrame(xdat)
 
@@ -103,7 +83,7 @@ npindexbw.default <-
     if (ncol(xdat) < 2) {
       if (coarseclass(xdat[,1]) != "numeric")
         stop("xdat must contain at least one continuous variable")
-      
+
       warning(paste("xdat has one dimension. Using a single index model to reduce",
                     "dimensionality is unnecessary."))
     }
@@ -112,54 +92,23 @@ npindexbw.default <-
       stop(paste("manually specified 'bws' must be a numeric vector of length ncol(xdat)+1.",
                  "See documentation for details."))
 
-###
-    ## first grab dummy args for scbandwidth() and perform 'bootstrap'
-    ## bandwidth call
+    tbw <- sibandwidth(beta = bws[1:ncol(xdat)],
+                       h = bws[ncol(xdat)+1], ...,
+                       nobs = dim(xdat)[1],
+                       xdati = untangle(xdat),
+                       ydati = untangle(data.frame(ydat)),
+                       xnames = names(xdat),
+                       ynames = deparse(substitute(ydat)),
+                       bandwidth = bws[ncol(xdat)+1],
+                       bandwidth.compute = bandwidth.compute)
 
-    mc.names <- names(match.call(expand.dots = FALSE))
-    margs <- c("ckertype", "ckerorder")
-
-    m <- match(margs, mc.names, nomatch = 0)
-    any.m <- any(m != 0)
-
-    tbw <- eval(parse(text=paste("sibandwidth(beta = bws[1:ncol(xdat)]",
-                        ifelse(any.m, ",",""),
-                        paste(mc.names[m], ifelse(any.m,"=",""), mc.names[m], collapse=", "),
-                        ", h = bws[ncol(xdat)+1],",
-                        "...,",
-                        "nobs = dim(xdat)[1],",
-                        "xdati = untangle(xdat),",
-                        "ydati = untangle(data.frame(ydat)),",
-                        "xnames = names(xdat),",
-                        "ynames = deparse(substitute(ydat)),",
-                        "bandwidth = bws[ncol(xdat)+1],",
-                        "bandwidth.compute = bandwidth.compute)")))
-    
-#    tbw <- sibandwidth(beta = bws[1:ncol(xdat)],
-#                       h = bws[ncol(xdat)+1], ...,
-#                       nobs = dim(xdat)[1],
-#                       xdati = untangle(xdat),
-#                       ydati = untangle(data.frame(ydat)),
-#                       xnames = names(xdat),
-#                       ynames = deparse(substitute(ydat)),
-#                       bandwidth = bws[ncol(xdat)+1],
-#                       bandwidth.compute = bandwidth.compute)
-
-    ## next grab dummies for actual bandwidth selection and perform call
-    if (tbw$method == "kleinspady" & !setequal(ydat,c(0,1))) 
+    if (tbw$method == "kleinspady" & !setequal(ydat,c(0,1)))
       stop("Klein and Spady's estimator requires binary ydat with 0/1 values only")
 
     mc.names <- names(match.call(expand.dots = FALSE))
-    margs <- c("nmulti",
-               "random.seed",
-               "optim.method",
-               "optim.maxattempts",
-               "optim.reltol",
-               "optim.abstol",
-               "optim.maxit",
-               "ckertype",
-               "ckerorder")
-    
+    margs <- c("nmulti","random.seed", "optim.method", "optim.maxattempts",
+               "optim.reltol", "optim.abstol", "optim.maxit", "only.optimize.beta")
+
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
@@ -176,21 +125,16 @@ npindexbw.default <-
     return(tbw)
   }
 
-  
 npindexbw.sibandwidth <-
   function(xdat = stop("training data xdat missing"),
            ydat = stop("training data ydat missing"),
-           bws,
-           bandwidth.compute = TRUE,
-           nmulti,
-           random.seed = 42,
+           bws, bandwidth.compute = TRUE, nmulti, random.seed = 42,
            optim.method = c("Nelder-Mead", "BFGS", "CG"),
            optim.maxattempts = 10,
            optim.reltol = sqrt(.Machine$double.eps),
            optim.abstol = .Machine$double.eps,
            optim.maxit = 500,
-           ckertype = c("gaussian", "epanechnikov","uniform"), 
-           ckerorder = c(2,4,6,8),           
+           only.optimize.beta = FALSE,
            ...){
 
     ## Save seed prior to setting
@@ -209,32 +153,20 @@ npindexbw.sibandwidth <-
     if (missing(nmulti)){
       nmulti <- min(5,ncol(xdat))
     }
-    
-    if (bws$method == "kleinspady" & !setequal(ydat,c(0,1))) 
+
+    if (bws$method == "kleinspady" & !setequal(ydat,c(0,1)))
       stop("Klein and Spady's estimator requires binary ydat with 0/1 values only")
 
     if (ncol(xdat) < 2) {
       if (coarseclass(xdat[,1]) != "numeric")
         stop("xdat must contain at least one continuous variable")
-      
+
       warning(paste("xdat has one dimension. Using a single index model to reduce",
                     "dimensionality is unnecessary."))
     }
 
-    ckertype = match.arg(ckertype)
-
-    if(missing(ckerorder))
-      ckerorder = 2
-    else if (ckertype == "uniform")
-      warning("ignoring kernel order specified with uniform kernel type")
-    else {
-      kord = eval(formals()$ckerorder) 
-      if (!any(kord == ckerorder))
-        stop("ckerorder must be one of ", paste(kord,collapse=" "))
-    }
-
     optim.method <- match.arg(optim.method)
-    
+
     ## catch and destroy NA's
     goodrows = 1:dim(xdat)[1]
     rows.omit = attr(na.omit(data.frame(xdat,ydat)), "na.action")
@@ -245,17 +177,17 @@ npindexbw.sibandwidth <-
 
     xdat = xdat[goodrows,,drop = FALSE]
     ydat = ydat[goodrows]
-    
+
     ## convert to numeric
     if (is.factor(ydat))
       ydat <- dlev(ydat)[as.integer(ydat)]
     else
       ydat <- as.double(ydat)
-    
+
     xdat = toMatrix(xdat)
 
     if(bandwidth.compute){
-      
+
       ## Note - there are two methods currently implemented, Ichimura's
       ## least squares approach and Klein and Spady's likelihood approach.
 
@@ -266,8 +198,7 @@ npindexbw.sibandwidth <-
       ##c(1,beta) which can be used in the index.model function above.
 
       ichimuraMaxPenalty <- 10*mean(ydat^2)
-      ichimuraFloor <- sqrt(.Machine$double.eps)
-      
+
       ichimura <- function(param) {
 
         ##Define the leave-one-out objective function, sum (y - \hat
@@ -275,17 +206,17 @@ npindexbw.sibandwidth <-
         ## parameters in `param') and then let h denote the kth column.
         if (ncol(xdat) == 1)
           beta = numeric(0)
-        else 
+        else
           beta <- param[1:(ncol(xdat)-1)]
-        
+
         h <- param[ncol(xdat)]
 
         ## Next we define the sum of squared leave-one-out residuals
 
-        sum.squares.leave.one.out <- function(xdat,ydat,beta,h,ckertype,ckerorder) {
+        sum.squares.leave.one.out <- function(xdat,ydat,beta,h) {
 
           ## Normalize beta_1 = 1 hence multiply X by c(1,beta)
-          
+
           index <- as.matrix(xdat) %*% c(1,beta)
 
           ## One call to npksum to avoid repeated computation of the
@@ -299,12 +230,10 @@ npindexbw.sibandwidth <-
                         leave.one.out=TRUE,
                         bandwidth.divide=TRUE,
                         bws=c(h),
-                        ckertype=ckertype,
-                        ckerorder=ckerorder)$ksum
+                        ckertype = bws$ckertype,
+                        ckerorder = bws$ckerorder)$ksum
 
-          denom <- tww[2,2,]
-          denom[which(denom == 0.0)] <- ichimuraFloor
-          fit.loo <- tww[1,2,]/denom
+          fit.loo <- tww[1,2,]/NZD(tww[2,2,])
 
           t.ret <- mean((ydat-fit.loo)^2)
           return(t.ret)
@@ -315,13 +244,15 @@ npindexbw.sibandwidth <-
         ## return an infinite penalty for negative h
 
         if(h > 0) {
-          return(sum.squares.leave.one.out(xdat=xdat,ydat=ydat,beta=beta,h=h,ckertype=ckertype,ckerorder=ckerorder))
+          return(sum.squares.leave.one.out(xdat,ydat,beta,h))
         } else {
           return(ichimuraMaxPenalty)
         }
 
       }
 
+      ichimura.nobw <- function(param,h){ return(ichimura(c(param,h))) }
+
       ## We define ichimura's objective function. Since we normalize beta_1
       ## to equal 1, we only worry about beta_2...beta_k in the index
       ## function for these internals, and when computing the leave-one-out
@@ -337,17 +268,17 @@ npindexbw.sibandwidth <-
         ## parameters in `param') and then let h denote the kth column.
         if (ncol(xdat) == 1)
           beta = numeric(0)
-        else 
+        else
           beta <- param[1:(ncol(xdat)-1)]
 
         h <- param[ncol(xdat)]
 
         ## Next we define the sum of logs
 
-        sum.log.leave.one.out <- function(xdat,ydat,beta,h,ckertype,ckerorder) {
+        sum.log.leave.one.out <- function(xdat,ydat,beta,h) {
 
           ## Normalize beta_1 = 1 hence multiply X by c(1,beta)
-          
+
           index <- as.matrix(xdat) %*% c(1,beta)
 
           ## One call to npksum to avoid repeated computation of the
@@ -361,12 +292,10 @@ npindexbw.sibandwidth <-
                         leave.one.out=TRUE,
                         bandwidth.divide=TRUE,
                         bws=c(h),
-                        ckertype=ckertype,
-                        ckerorder=ckerorder)$ksum
+                        ckertype = bws$ckertype,
+                        ckerorder = bws$ckerorder)$ksum
 
-          denom <- tww[2,2,]
-          denom[which(denom == 0.0)] <- kleinspadyFloor
-          ks.loo <- tww[1,2,]/denom
+          ks.loo <- tww[1,2,]/NZD(tww[2,2,])
 
           ## Avoid taking log of zero (ks.loo = 0 or 1 since we take
           ## the log of ks.loo and the log of 1-ks.loo)
@@ -384,7 +313,7 @@ npindexbw.sibandwidth <-
         ## return an infinite penalty for negative h
 
         if(h > 0) {
-          return(sum.log.leave.one.out(xdat=xdat,ydat=ydat,beta=beta,h=h,ckertype=ckertype,ckerorder=ckerorder))
+          return(sum.log.leave.one.out(xdat,ydat,beta,h))
         } else {
           ## No natural counterpart to var of y here, unlike Ichimura above...
           return(sqrt(.Machine$double.xmax))
@@ -392,6 +321,7 @@ npindexbw.sibandwidth <-
 
       }
 
+      kleinspady.nobw <- function(param,h){ return(kleinspady(c(param,h))) }
       ## Now we implement multistarting
 
       fval.min <- .Machine$double.xmax
@@ -401,22 +331,20 @@ npindexbw.sibandwidth <-
       console <- newLineConsole()
 
       if(bws$method == "ichimura"){
-        optim.fn <- ichimura
-        optim.control <- c(abstol=optim.abstol,
-                           reltol=optim.reltol,
-                           maxit=optim.maxit)
+        optim.fn <- if(only.optimize.beta) ichimura.nobw else ichimura
+        optim.control <- list(abstol=optim.abstol,
+                              reltol=optim.reltol,
+                              maxit=optim.maxit)
       } else if(bws$method == "kleinspady"){
-        optim.fn <- kleinspady
-        optim.control <- c(abstol=optim.abstol, ## added jracine jan 20 2011 (was in ichimura but not kleinspady)
-                           reltol=optim.reltol,
-                           maxit=optim.maxit)
+        optim.fn <- if(only.optimize.beta) kleinspady.nobw else  kleinspady
+        optim.control <- list(reltol=optim.reltol,maxit=optim.maxit)
       }
 
       for(i in 1:nmulti) {
 
         console <- printPush(paste(sep="", "Multistart ", i, " of ", nmulti, "..."), console)
         ##cv.console <- newLineConsole(console)
-        
+
         n <- nrow(xdat)
 
         ## We use the nlm command to minimize the objective function using
@@ -429,63 +357,66 @@ npindexbw.sibandwidth <-
           ## multistart 1
           ols.fit <- lm(ydat~xdat,x=TRUE)
           fit <- fitted(ols.fit)
-          
+
           if (ncol(xdat) != 1){
             if (setequal(bws$beta[2:ncol(xdat)],c(0)))
-              beta <- coef(ols.fit)[3:ncol(ols.fit$x)]/(coef(ols.fit)[2])
+              beta <- coef(ols.fit)[3:ncol(ols.fit$x)]
             else
               beta = bws$beta[2:ncol(xdat)]
           } else { beta = numeric(0) }
 
           if (bws$bw == 0)
-            if(IQR(fit) > 0) {
-              h <- (4/3)^0.2*min(sd(fit),IQR(fit)/(qnorm(.25,lower.tail=F)*2))*n^(-1/5)
-            } else {
-             h <- (4/3)^0.2*sd(fit)*n^(-1/5)
-            }
+            h <- 1.059224*EssDee(fit)*n^(-1/5)
           else
             h <- bws$bw
         } else {
           ## Random initialization used for remaining multistarts
 
           beta.length <- length(coef(ols.fit)[3:ncol(ols.fit$x)])
-          beta <- runif(beta.length,min=0.5,max=1.5)*coef(ols.fit)[3:ncol(ols.fit$x)]/(coef(ols.fit)[2])
-          if(IQR(fit) > 0) {
-            h <- runif(1,min=0.5,max=1.5)*min(sd(fit),IQR(fit)/(qnorm(.25,lower.tail=F)*2))*n^(-1/5)
-          } else {
-            h <- runif(1,min=0.5,max=1.5)*sd(fit)*n^(-1/5)
-          }
+          beta <- runif(beta.length,min=0.5,max=1.5)*coef(ols.fit)[3:ncol(ols.fit$x)]
+          if (!only.optimize.beta)
+              h <- runif(1,min=0.5,max=1.5)*EssDee(fit)*n^(-1/5)
         }
-        
-        suppressWarnings(optim.return <- optim(c(beta,h),fn=optim.fn,gr=NULL,method=optim.method,control=optim.control))
+
+        optim.parm <- if(only.optimize.beta) beta else c(beta,h)
+
+        topt <- parse(text=paste("optim(optim.parm,fn=optim.fn,gr=NULL,method=optim.method,control=optim.control", ifelse(only.optimize.beta, ',h)',')')))
+        suppressWarnings(optim.return <- eval(topt))
         attempts <- 0
         while((optim.return$convergence != 0) && (attempts <= optim.maxattempts)) {
           attempts <- attempts + 1
           beta.length <- length(coef(ols.fit)[3:ncol(ols.fit$x)])
-          beta <- runif(beta.length,min=0.5,max=1.5)*coef(ols.fit)[3:ncol(ols.fit$x)]/(coef(ols.fit)[2])
-          if(IQR(fit) > 0) {
-            h <- runif(1,min=0.5,max=1.5)*min(sd(fit),IQR(fit)/(qnorm(.25,lower.tail=F)*2))*n^(-1/5)
-          } else {
-            h <- runif(1,min=0.5,max=1.5)*sd(fit)*n^(-1/5)
+          beta <- runif(beta.length,min=0.5,max=1.5)*coef(ols.fit)[3:ncol(ols.fit$x)]
+          if(!only.optimize.beta)
+              h <- runif(1,min=0.5,max=1.5)*EssDee(fit)*n^(-1/5)
+
+          if(optim.return$convergence == 1){
+              if(optim.control$maxit < (2^32/10))
+                  optim.control$maxit <- 10*optim.control$maxit
+              else
+                  stop(paste("optim failed to converge after optim.maxattempts = ", optim.maxattempts, " iterations."))
           }
-          optim.control <- lapply(optim.control,'*',10.0)
-          suppressWarnings(optim.return <- optim(c(beta,h),fn=optim.fn,gr=NULL,method=optim.method,control=optim.control))
+
+          if(optim.return$convergence == 10){
+              optim.control$reltol <-  10.0*optim.control$reltol
+              if(!is.null(optim.control$abstol))
+                  optim.control$abstol <-  10.0*optim.control$abstol
+          }
+          
+          suppressWarnings(optim.return <- eval(topt))
         }
-        
 
         if(optim.return$convergence != 0)
           stop(paste("optim failed to converge after optim.maxattempts = ", optim.maxattempts, " iterations."))
 
-
-        fval.value[i] <- optim.return$value       
-        if(optim.return$value < fval.min) {        
-          param <- optim.return$par
+        fval.value[i] <- optim.return$value
+        if(optim.return$value < fval.min) {
+          param <- if(only.optimize.beta) c(optim.return$par,h) else optim.return$par
           fval.min <- optim.return$value
           numimp <- numimp + 1
           best <- i
         }
 
-        
         if(i != nmulti)
           console <- printPop(console)
       }
@@ -498,8 +429,6 @@ npindexbw.sibandwidth <-
       bws$ifval <- best
       bws$numimp <- numimp
       bws$fval.vector <- fval.value
-      bws$ckertype <- ckertype
-      bws$ckerorder <- ckerorder
     }
     ## Return a list with beta (we append the restricted value of
     ## beta_1=1), the bandwidth h, the value of the objective function at
@@ -507,10 +436,6 @@ npindexbw.sibandwidth <-
     ## value of the objective function, the restart that resulted in the
     ## smallest value, and the vector of objective function values.
 
-    ## TRISTEN XXX - kindly make sure I have named return vals consistent
-    ## with other functions, e.g., ifval, fval etc? Also, kindly return a
-    ## `pretty print' type of object
-
     ## Restore seed
 
     if(exists.seed) assign(".Random.seed", save.seed, .GlobalEnv)
@@ -532,8 +457,8 @@ npindexbw.sibandwidth <-
                        bandwidth = bws$bw,
                        rows.omit = rows.omit,
                        bandwidth.compute = bandwidth.compute,
-                       optim.method = optim.method)
-
+                       optim.method = optim.method,
+                       only.optimize.beta = only.optimize.beta)
 
     bws
 
